/* W. Serwe --  -- 1.2 -- 2016:15:30 16:15:30 */
/* correct implementation of the specification from [Jard-Jeron-05] */

#define _DEFAULT_SOURCE /* for random() */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

/* ------------------------------------------------------------------------ */

static char *line;
static size_t size;
static int verbose;

/* ------------------------------------------------------------------------ */

void error (char *message)
{
     (void) fprintf (stderr, "%s\n", message);
     exit (1);
}

/* ------------------------------------------------------------------------ */

char read ()
{
     ssize_t n;

     errno = 0;
     n = getline (&line, &size, stdin);
     if (n < 0) {
	  if (errno == 0) {
	       if (verbose)
		    fprintf (stderr, "end of input\n");
	       exit (0);
	  } else
	       error ("input failure");
     }
     if ((n < 1) || (n > 2)) {
	  fprintf (stderr, "\"%s\": ", line);
	  error ("incorrect input");
     }
     if (verbose)
	  fprintf (stderr, "read \"%c\"\n", line[0]);

     switch (line[0]) {
     case 'A':
     case 'B':
     case 'C':
	  return (line[0]);
     default:
	  error ("unsupported input");
	  /* NOTREACHED */
	  return (0);
     }
}

/* ------------------------------------------------------------------------ */

void write (char c)
{
     fprintf (stdout, "%c\n", c);
     fflush (stdout);
}

/* ------------------------------------------------------------------------ */

typedef enum {s0, s1, s2, s3, s4, s5, s6} state;

void automaton (int quiescence /* 0 or 1 */) {
     state current;

     current = s0;

     while (1) {
	  if (verbose) {
	       fprintf (stderr, "current state is %d\n", current);
	  }
	  switch (current) {
	  case s0:
	       switch (read ()) {
	       case 'A':
		    current = s1;
		    break;
	       case 'B':
		    current = s2;
		    break;
	       case 'C':
		    current = s3;
		    break;
	       }
	       break;

	  case s1:
	       if (random () % 2 == 0) {
		    write ('X');
		    current = s4;
	       } else {
		    write ('Y');
		    current = s0;
	       }
	       break;

	  case s2:
	       switch (random () % (2 + quiescence)) {
	       case 0:
		    write ('Y');
		    current = s0;
		    break;
	       case 1:
		    write ('Z');
		    current = s0;
		    break;
	       default:
		    /* nothing */
		    break;
	       }
	       break;

	  case s3:
	       write ('Y');
	       current = s5;
	       break;

	  case s4:
	       switch (read ()) {
	       case 'A':
		    current = s6;
		    break;
	       default:
		    error ("unexpected input");
	       }
	       break;

	  case s5:
	       switch (read ()) {
	       case 'C':
		    current = s3;
		    break;
	       default:
		    error ("unexpected input");
	       }
	       break;

	  case s6:
	       write ('X');
	       current = s4;
	       break;
	  }
     }
}

/* ------------------------------------------------------------------------ */

int main (int argc, char **argv)
{
     int quiescence = 0;
     
     line = NULL;
     size = 0;
     verbose = 0;

     /* ignore program name */
     argc--;
     argv++;

     while (argc > 0) {
	  if (strcmp (argv[0], "-quiescence") == 0) {
	       quiescence = 1;
	       argc--;
	       argv++;
	  } else if (strcmp (argv[0], "-seed") == 0) {
	       if (argc < 2)
		    error ("missing value for -seed");
	       srandom (atoi (argv[1]));
	       argc -= 2;
	       argv += 2;
	  } else if (strcmp (argv[0], "-verbose") == 0) {
	       verbose = 1;
	       argc--;
	       argv++;
	  } else {
	       fprintf (stderr, "\"%s\": ", argv[0]); 
	       error ("unsupported option");
	  }
     }
     
     automaton (quiescence);
     
     free (line);
}

/* ------------------------------------------------------------------------ */

