.\" @(#)bcg_control.l - INRIA Rhone-Alpes - 2018/11/12
.TH CADP LOCAL "2018/11/12" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0
.SH NAME
bcg_control \- extract test cases from a complete test graph
 
.SH SYNOPSIS

.B bcg_control
[\fB-all\fP|\fB-random\fP]
[\fB-count\fP]
[\fB-delta\fP|\fB-fail\fP|\fB-inconclusive\fP|\fB-pass\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP]
[\fB-decorate\fP]
[\fB-keep\fP]
[\fB-monitor\fP]
[\fB-parse\fP|\fB-unparse\fP]
[\fB-seed\fP \fIN\fP]
[\fB-stacksize\fP \fIN\fP]
[\fB-strong\fP|\fB-weak\fP]
[\fB-verbose\fP]
[\fB-version\fP]
\fB-input\fP|\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
\fIctg\fP[\fB.bcg\fP]
\fItc\fP[\fB.bcg\fP]
.P

.SH DESCRIPTION
\fBbcg_control\fP extracts a concrete test case \fItc\fP\fB.bcg\fP from a CTG \fIctg\fP\fB.bcg\fP.
Both files are stored in the
.BR bcg (LOCAL)
format.

See the
.BR testor (LOCAL)
manual page for a description of the match file format.
The default matching option is \fB-total\fP.

.SH B\&C\&G FORMAT
See the
.BR bcg (LOCAL)
manual page.

.SH OPTIONS
The following options are currently available:
.TP
.B -all\ \ \ 
Generate all test cases that can be extracted from the complete test graph \fIctg\fP\fB.bcg\fP. The test cases are written to files named "\fItc\fP\fB-N.bcg\fP", where the number \fBN\fP is incremented for each new test case.
Mutually exclusive with option "\fB-random\fP".
Not a default option.
.TP
\fB-count\ \ \ 
Only count and display the number of test cases, do not generate the corresponding \fBBCG\fP files.
This option is only useful with option "\fB-all\fP" (set automatically).
Not a default option.
.TP
\fB-delta\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the labels tagging quiescent states according to the match file \fIfile\fP.
If this option is absent, the label ":DELTA:" is used to identify quiescent states.
.TP
\fB-fail\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the labels tagging fail states according to the match file \fIfile\fP.
If this option is absent, the label ":FAIL:" is used to identify fail states.
.TP
\fB-inconclusive\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the labels tagging inconclusive states according to the match file \fIfile\fP.
If this option is absent, the label ":INCONCLUSIVE:" is used to identify inconclusive states.
.TP
\fB-input\fP|\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the inputs of the specification (outputs of the tester) according to the match file \fIfile\fP.
Mandatory option.
.TP
.B -decorate\ \ \ \ \ 
Annotate transition labels of the test case to indicate if they correspond to inputs or outputs from the point of view of the tester (i.e., opposite to the point of view expressed in the input match file).
Not a default option.
.TP
.B -keep\ \ \ \ \ \ \ 
When generating a test case, keep the state numbers of the complete test graph. This helps understanding the selection process, but yields a less efficient encoding as BCG, because there might be unused state numbers.
Not a default option.
.TP
.B -monitor\ \ \ \ \ \ \ \ \ 
Open a window for monitoring in real-time the generation of each test case.
Not a default option.
.TP
.B -parse\ \ \ \ \ \ \ \ \ 
Enable label parsing in the generated test case.
See the
.BR bcg (LOCAL)
manual page for information about label parsing.
Default option.
.TP
\fB-pass\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the labels tagging accepting states according to the match file \fIfile\fP.
If this option is absent, the label ":PASS:" is used to identify accepting states.
.TP
.B -random \ \ \
Use a random number generator when choosing between different ways to solve a controllability conflict.
Mutually exclusive with option "\fB-all\fP".
Not a default option.
.TP
.BI -seed \ N \ \ 
Use \fIN\fP as seed for the random number generator.
This option is only useful in combination with option "\fB-random\fP" (set automatically).
.TP
.BI -stacksize \ N \ \ 
Use \fIN\fP as size of the exploration stack.
This option is not useful in combination with option "\fB-random\fP".
Default size is 42. 
.TP
.BI -strong \ \ 
Enforce controllability according to the definition of [JJ05].
Default option.
.TP
.B -unparse\ \ \ \ 
Disable label parsing in the generated test case.
See the
.BR bcg (LOCAL)
manual page for information about label parsing.
Not a default option.
.TP
.BR -verbose\ \ \ 
Report in detail activities and progress to the user's screen.
Not a default option.
.TP
.BR -version\ \ \ 
Display the version number and stop.
Not a default option.
.TP
.BI -weak \ \ 
Enforce weaker controllability than the definition of [JJ05], by allowing a choice between inputs and outputs, but still prohibiting a choice between outputs (from the point of view of the tester, i.e., pposite to the point of view expressed in the input match file).
Not a default option.

.SH OPERANDS
.PD 0
.TP 11
.IB ctg .bcg
complete test graph in the \fBbcg\fP format (input)
.TP
.IB tc .bcg
test case in the \fBbcg\fP format (output)
.TP
.I file
rules characterizing input labels (input)

.SH EXIT STATUS
Exit status is 0 if everything is alright, 1 otherwise.

.SH AUTHORS
Hubert Garavel, Radu Mateescu, and Wendelin Serwe.

.SH BIBLIOGRAPHY

.TP 8
[JJ05]
Claude Jard and Thierry Jeron.
TGV: theory, principles and algorithms.
International Journal on Software Tools for Technology Transfer 7(4):297-315, August 2005.

.SH SEE ALSO
.P
.BR bcg (LOCAL),
.BR testor (LOCAL)

.P
Additional information is available from the CADP Web page located at
http://cadp.inria.fr
.P
Directives for installation are given in files \fB$CADP/INSTALLATION_*\fP.
.P
Recent changes and improvements to this software are reported
and commented in file \fB$CADP/HISTORY\fP.

.SH BUGS
Please report any bug to cadp@inria.fr

