.\" @(#)combinator.open.l - INRIA Rhone-Alpes - 2020/02/12
.TH CADP LOCAL "2020/02/12" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0
.SH NAME
sync.open \- OPEN/CAESAR connection for the synchronous product of a specification and a test purpose

.SH SYNOPSIS
.BR sync.open
[\fB-cc\fP \fIcc-options\fP]
[\fB-silent\fP | \fB-verbose\fP]
[\fB-debug\fP]
[\fB-force\fP]
[\fB-accept\fP [\fB-total\fP|\fB-partial\fP|\fB-gate\fP] \fIfile\fP]
[\fB-refuse\fP [\fB-total\fP|\fB-partial\fP|\fB-gate\fP] \fIfile\fP]
[\fB-otherwise\fP [\fB-total\fP|\fB-partial\fP|\fB-gate\fP] \fIfile\fP]
[\fB-otherwise-matches-tau\fP]
[\fB-purpose\fP \fIprocess-instantiation\fP]
[\fB-spec\fP \fIprocess-instantiation\fP]
\fIspecification\fP[.\fBbcg\fP|.\fBlnt\fP|.\fBlot\fP|.\fBlotos\fP]
\fIpurpose\fP[.\fBbcg\fP|.\fBlnt\fP|.\fBlot\fP|.\fBlotos\fP]
\fIprog\fP[\fB.a\fP|\fB.c\fP|\fB.o\fP]
.RI [ prog_options ]

.SH DESCRIPTION
Taking as input a specification
.I specification, 
a test purpose
.I purpose,
and an OPEN/CAESAR program
\fIprog\fP[\fB.a\fP|\fB.c\fP|\fB.o\fP],
\fBsync.open\fP applies the \fIprog\fP tool to the synchronous product (computed on the fly) of \fIspecification\fP and \fIpurpose\fP.
Both, \fIspecification\fP and \fIpurpose\fP are an LNT model, a LOTOS specification, or an explicit LTS in the BCG format.

The test purpose may contain special labels, called "accept", "refuse", and "otherwise" labels.
The decision is whether a given label \fIl\fP is a special label uses regular expressions contained in match files provided with the options \fB-accept\fP, \fB-refuse\fP, and \fBotherwise\fP (see below for details).

The test purpose has to be deterministic.
Foreach state \fIs\fP of the test purpose with an outgoing "accept" or "refuse" transition \fIt\fP, this transition \fIt\fP should be the only outgoing transition of \fIs\fP.

\fBsync.open\fP constructs the implementation of the OPEN/CAESAR graph module corresponding to the synchronous product in several steps.
First it generates the C code for \fIspecification\fP (respectively, for \fIpurpose\fP), using the \fBlnt2lotos\fP, \fBcaesar.adt\fP, and \fBcaesar\fP tools.
Then, it modifies the generated C code, adding a suffix ("_1" for the specification and "_2" for the test purpose) to all the C types, functions, and macros corresponding to the implementation of the OPEN/CAESAR graph module.
Finally, it compiles the modified C code and links it with the library sync.a, which implements the generic functions computing the synchronous product.

The synchronous product is defined as follows.
The set of states of the synchronous product is the set of couples \fI<s1,s2>\fP, where \fIs1\fP is a state of the specification and \fIs2\fP is a state of the test purpose, plus a new initial state.
The set of labels of the synchronous product is the set of labels of the specification, plus the particular label "TESTOR_DELTA" and the set of labels of the test purpose corresponding to "accept" and "refuse" labels.

The initial state of the product has a single outgoing transition labeled with "TESTOR_DELTA" leading to the state \fI<si1,si2>\fP, where \fIsi1\fP (respectively, \fIsi2\fP) is the initial state of the specification (respectively, the test purpose).
For each state \fI<s1,s2>\fP of the product (different from the initial state), the set of outgoing transitions is computed by applying the first applicable step out of the following.

.TP 3
a)
If the test purpose contains a transition labeled with \fIl\fP from \fIs2\fP to a state \fIs2'\fP, where \fIl\fP corresponds to either an "accept" or a "refuse" label, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1,s2'>\fP is added to the product.

Note: more precisely, due to optimisations, the transition in the product is labeled with \fIl'\fP, where \fIl'\fP corresponds to an "accept" label if \fIl\fP corresponds to an "accept" label, and where \fIl'\fP corresponds to a "refuse" label if \fIl\fP corresponds to a "refuse" label.

.TP
b)
If the specification contains a transition labeled with \fIl\fP from \fIs1\fP to a state \fIs1'\fP, and the test purpose contains a transition labeled with \fIl\fP from \fIs2\fP to a state \fIs2'\fP, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1',s2'>\fP is added to the product.

Note that the label \fIl\fP can also be the hidden internal label \fBi\fP: this enables to synchronize the internal transitions of the specification and the test purpose.

.TP
c)
If the specification contains a transition labeled with \fIl\fP (different from the internal label \fBi\fP) from \fIs1\fP to a state \fIs1'\fP, and the test purpose contains a transition labeled with \fIl'\fP from \fIs2\fP to a state \fIs2'\fP, where \fIl'\fP corresponds to an "otherwise" label, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1',s2'>\fP is added to the product.

.TP 
d)
If the specification contains a transition labeled with \fIl\fP from \fIs1\fP to a state \fIs1'\fP, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1',s2>\fP is added to the product.

.TP 0
\ 
Classically, \fBsync.open\fP is used in combination with the \fBtestor\fP tool to extract a complete test graph for a test purpose from a specification.
See
.BR lotos.open (LOCAL),
.BR caesar.adt (LOCAL),
.BR caesar (LOCAL),
.BR lnt.open (LOCAL),
and
.BR testor (LOCAL)
for more information.
.br

.SH OPTIONS
.TP
\fB-accept\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
The matching rules of \fIfile\fP are used to characterize labels to tag accepting states of the test purpose.
If this option is absent, the file "combinator_accept" must exist and is used.
In any case, the default matching option is \fB-total\fP.
.TP
.BI -cc \ cc-options
Add \fIcc-options\fP to the options passed to the C compiler.
.TP
.B -debug
Do not remove intermediate files generated during the translation to C.
Not a default option.
.TP
.B -force
Force regeneration and compilation of LOTOS and/or C code.
Not a default option.
.TP
\fB-otherwise\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
The matching rules of \fIfile\fP are used to characterize labels matching "otheriwse" transitions of the test purpose.
If this option is absent, the file "combinator_otherwise" must exist and is used.
In any case, the default matching option is \fB-total\fP.
.TP
\fB-otherwise-matches-tau\fP
If this option is present, an "otherwise" transition of the test purpose matches also internal, hidden transitions of the specification.
Concretely, this option removes the constraint "\fIl\fP different from the internal label \fBi\fP" from step c) above.
Not a default option.
.TP
\fB-purpose\fP \fIprocess-instantiation\fP
Use process \fIprocess-instantiation\fP as the principal process for the test purpose.
Default principal process is \fIMAIN\fP.
This option is only meaningful for a test purpose in LNT or LOTOS.
.TP
\fB-refuse\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
The matching rules of \fIfile\fP are used to characterize labels to tag refuse states of the test purpose.
If this option is absent, the file "combinator_refuse" must exist and is used.
In any case, the default matching option is \fB-total\fP.
.TP
.B -silent
Execute silently.
This option is passed to \fBlnt.open\fP and \fBlotos.open\fP.
See
.BR lnt.open (LOCAL)
and
.BR lotos.open (LOCAL)
for more information.
.TP
\fB-spec\fP \fIprocess-instantiation\fP
Use process \fIprocess-instantiation\fP as the principal process for the specification.
Default principal process is \fIMAIN\fP.
This option is only meaningful for a specification in LNT or LOTOS.
.TP
.B -verbose
Report in detail activities and progress, including errors and messages emitted by auxiliary tools, to the user's screen.
This option is passed to \fBlnt.open\fP and \fBlotos.open\fP.
See
.BR lnt.open (LOCAL)
and
.BR lotos.open (LOCAL)
for more information.

.TP
.I prog_options
if any, are passed to \fIprog\fP.

.SH OPERANDS
.PD 0
.TP 20
.IB specification .lnt
LOTOS NT specification (input)
.TP
.IB specification .lot
LOTOS specification (input)
.TP
.IB specification .lotos
LOTOS specification (input)
.TP
.IB purpose .lnt
LOTOS NT test purpose (input)
.TP
.IB purpose .lot
LOTOS test purpose (input)
.TP
.IB purpose .lotos
LOTOS test purpose (input)
.TP
.IB specification .o
object code (output)
.TP
.IB purpose .o
object code (output)
.TP
.IB specification .err
detailed error messages (output)
.TP
.IB purpose .err
detailed error messages (output)
.TP
.IB prog .a
exploration module (archive, input)
.TP
.IB prog .c
exploration module (source, input)
.TP
.IB prog .o
exploration module (object code, input)
.TP
.I prog
executable program (output)

.SH FILES
.PD 0
.TP 20
.B $CADP_TMP/lnt.*
temporary files
.TP
.B $CADP/lib/LNT_V1.lib
LOTOS NT predefined library (LOTOS code)
.TP
.B $CADP/incl/LNT_V1.h 
LOTOS NT predefined library (C code)

.SH ENVIRONMENT_VARIABLES
Technically, the options of \fBsync.open\fP are passed to the library sync.a through the following environment variables: if these variables are defined before calling \fBsync.open\fP, the behaviour is undefined.
.TP 34
COMBINATOR_ACCEPT_FILENAME
.TP
COMBINATOR_ACCEPT_KIND
.TP
COMBINATOR_BITMAP_SIZE
.TP
COMBINATOR_LABEL_TABLE_SIZE
.TP
COMBINATOR_OTHERWISE_FILENAME
.TP
COMBINATOR_OTHERWISE_KIND
.TP
COMBINATOR_OTHERWISE_MATCHES_TAU
.TP
COMBINATOR_PURPOSE_FILENAME
.TP
COMBINATOR_REFUSE_FILENAME
.TP
COMBINATOR_REFUSE_KIND
.TP
COMBINATOR_SPECIFICATION_FILENAME
.TP
COMBINATOR_STATE_TABLE_SIZE
\ 

.SH EXIT STATUS
When the source is erroneous, error messages are issued.
Exit status is 0 if everything is alright, 1 otherwise.

.SH AUTHORS
.P
Hubert Garavel, Radu Mateescu, and Wendelin Serwe.

.SH SEE ALSO
.P
.BR bcg (LOCAL),
.BR lotos.open (LOCAL),
.BR lnt.open (LOCAL),
.BR testor (LOCAL)

.P
Additional information is available from the CADP Web page located at
http://cadp.inria.fr
.P
Directives for installation are given in files \fB$CADP/INSTALLATION_*\fP.
.P
Recent changes and improvements to this software are reported
and commented in file \fB$CADP/HISTORY\fP.

.SH BUGS
.P
Please report any bug to cadp@inria.fr
