.\" @(#)extract_all.l - INRIA Rhone-Alpes - 2020/08/27
.TH CADP LOCAL "2020/08/27" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0
.SH NAME

extract_all \- extract a set of test cases from a complete test graph
 
.SH SYNOPSIS

\fBextract_all\fP
[\fB-silent | -verbose\fP]
[\fB-first | -last\fP]
[\fB-monitor | -parse | -unparse\fP]
\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
\fIctg\fP[.\fBbcg\fP]
[\fItestcase\fP]

.SH DESCRIPTION

\fBextract_all\fP extracts from the complete test graph \fIctg\fP\fB.bcg\fP a set of test cases covering all choices to solve the controllability issues present in \fIctg\fP\fB.bcg\fP.
\fBextract_all\fP first computes the set of all these choices, and then iteratively extracts a test case handling one of the choices and updates the set of choices accordingly.
Test case extraction stops when there are no more choices to be solved.

The test cases are named \fItestcase\fP\fB.\fP\fIN\fP\fB.bcg\fP (if the last optional argument is present) or \fIctg\fP\fB.\fP\fIN\fP\fB.bcg\fP (if the last optional argument is absent); in both cases, \fIN\fP is the number of the extracted case, the test cases being numbered starting from 1.

Note that the extracted test cases are not necessarily sub-graphs of the complete test graph, because covering some choices might require to unfold cycles in the complete test graph.
This is a major difference of \fBextract_all\fP with 
.BR bcg_control (LOCAL).

\fBextract_all\fP is essentially a wrapper script for the
.BR testor (LOCAL)
and
.BR bcg_alternatives (LOCAL)
tools.

.SH OPTIONS

The following options are currently available:
.TP
\fB-base\fP \fIdirectory\fP
Use \fIdirectory\fP (rather than \fI$CADP_TEMPORARY\fP) for temporary files, such as prefixes or the set of alternatives.
If \fIdirectory\fP exists prior to the call to \fBextract_all\fP, it is not removed at the end of the execution.
Not a default option.
.TP
.B -check\ \ \ \ \ \ 
Eliminate all inaccessible states from the generated test cases and check the accessibility of a PASS verdict.
Not a default option.
.TP
.B -continue\ \ \ \ \ 
Start with the set of alternatives with the largest number present in the directory of temporary files.
This option requires the use of \fB-base\fP.
Not a default option.
.TP
.B -copy\ \ \ \ \ 
If \fIctg\fP\fB.bcg\fP is controllable, store an equivalent graph (possibly reduced by strong bisimulation and with states renumbered according to a breadth-first traversal) as first and only test case.
Not a default option.
.TP
.B -first\ \ \ \ \ 
For the selection of the next choice to handle, apply a heuristics selecting the first possible choice.
This option is passed to the
.BR bcg_alternatives (LOCAL)
tool.
Not a default option.
.TP
\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the inputs of the specification (outputs of the tester) according to the match file \fIfile\fP.
Mandatory option.
.TP
.B -last\ \ \ \ \ 
For the selection of the next choice to handle, apply a heuristics selecting the deepest possible choice.
This option is passed to the
.BR bcg_alternatives (LOCAL)
tool.
Default option.
.TP
.B -monitor
Open a window for monitoring in real-time the generation of each test case.
Not a default option.
.TP
.B -parse\ \ \ 
Enable label parsing in the extracted test cases.
See the
.BR bcg (LOCAL)
manual page for information about label parsing.
Default option.
.TP
.B -silent
Execute silently.
Not a default option.
.TP
.B -unparse
Disable label parsing in the extracted test cases.
See the
.BR bcg (LOCAL)
manual page for information about label parsing.
Not a default option.
.TP
.B -verbose
Report in detail activities and progress, including errors and messages emitted by auxiliary tools, to the user's screen.
Not a default option.

.SH EXIT STATUS
.P
Exit status is 0 if everything is alright, 1 otherwise.

.SH AUTHORS
.P
Radu Mateescu and Wendelin Serwe.

.SH SEE ALSO
.P
.BR bcg (LOCAL),
.BR bcg_alternatives (LOCAL),
.BR bcg_control (LOCAL),
.BR caesar_hide_1 (LOCAL),
.BR testor (LOCAL)

.P
Additional information is available from the CADP Web page located at
http://cadp.inria.fr
.P
Directives for installation are given in files \fB$CADP/INSTALLATION_*\fP.
.P
Recent changes and improvements to this software are reported
and commented in file \fB$CADP/HISTORY\fP.

.SH BUGS
.P
Please report any bug to cadp@inria.fr
