.\" @(#)extract_tests.l - INRIA Rhone-Alpes - 2020/02/12
.TH CADP LOCAL "2020/02/12" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0
.SH NAME

extract_tests \- compute test cases
 
.SH SYNOPSIS

\fBextract_tests\fP
[\fB-version\fP]
[\fB-silent | -verbose\fP]
[\fB-debug\fP]
[\fB-decorate\fP]
[\fB-force\fP]
[\fB-stat\fP]
[\fB-timer\fP]
[\fB-unlock\fP]
[\fB-cc\fP \fIcc-options\fP]
[\fB-monitor | -parse | -unparse\fP]
[\fB-ctg\fP \fItestgraph\fP]
[\fB-count\fP]
[\fB-all\fP|\fB-random\fP|\fB-seed\fP \fIN\fP]
[\fB-accept\fP|\fB-otherwise\fP|\fB-refuse\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP]
[\fB-purpose\fP \fIprocess-instantiation\fP]
[\fB-spec\fP \fIprocess-instantiation\fP]
\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
\fIspecification\fP[.\fBbcg\fP|.\fBlnt\fP|.\fBlot\fP|.\fBlotos\fP]
\fIpurpose\fP[.\fBbcg\fP|.\fBlnt\fP|.\fBlot\fP|.\fBlotos\fP]
[\fItestcase\fP[.\fBbcg\fP]]

.SH DESCRIPTION

\fBextract_tests\fP enables the generation of abstract test cases from a specification and a test purpose, both given as an LNT model, a LOTOS specification, or an explicit LTS in the BCG format.
\fBextract_tests\fP is essentially a wrapper script for the
.BR testor (LOCAL),
.BR combinator.open (LOCAL),
and
.BR bcg_control (LOCAL)
tools.

The specification and the test purpose are Inputs/Outputs Labeled Transition Systems (IOLTS). The set of labels corresponding to inputs is specified by an input/output (i/o) match file given with the mandatory option \fB-io\fP.

.SH TEST PURPOSE

A test purpose is a subgraph of the specification, so as to reduce the part of the specification to be explored during the test case generation.

Final states of the test purpose graph are either accepting states (the purpose is reached) or refusing states (parts of the specification are rejected and not explored further).
Accepting (respectively refusing) states must be given as loop-transitions with the predefined label \fBACCEPT\fP or \fBaccept\fP (respectively \fBREFUSE\fP or \fBrefuse\fP). The purpose must own one accepting state at least.

Labels matching the match file given with option \fB-otherwise\fP (called "otherwise labels") are used to complete the test purpose.
Concretely, each transition "\fIsource\fP --\fIlabel\fP--> \fItarget\fP" where \fIlabel\fP is an otherwise label, is replaced by a set of transitions "\fIsource\fP --\fIlabel_i\fP--> \fItarget\fP", such that the set of labels of all transitions with source state \fIsource\fP is equal to the set of actions of the specification.

.SH TEST CASE

If the test purpose is valid, i.e. allows to select sequences of the specification leading to an accepting state, \fBextract_tests\fP produces the a test case in the
.BR bcg (LOCAL)
format.
A test case is an IOLTS defining a tester, which interacts with a system under test (SUT) in order to drive the SUT towards an accepting state of the test purpose, if possible.

Actions labels of the test case are those of the specification, plus the special labels ":DELTA:" (corresponding to a lock transition), ":INCONCLUSIVE:" (corresponding to an inconclusive verdict), and ":PASS:" (corresponding to an accepting verdict).
An accepting verdict means that the tester has detected no implementation error in the implementation under test.
An inconclusive verdict means that the tester has reached a state not covered by the test purpose.
Fail verdicts are implicit: the implementation under test is not conform to the specification if the implementation outputs an action not included in the set of actions of the current state of the test graph.

.SH MATCH FILE FORMAT

Each match file characterizes a set of actions by means of a set of regular expressions (according to the definition given in the manual page of the \s-1POSIX\s+1
.BR regexp (LOCAL)),
according to the following grammar:
.ne 6v
.nf
.cs 4 23
.cs I 23

 \fB<i/o-match-file> ::= 'input' | 'output' \\n <regexp-list>

 \fB<match-file> ::= 'match' | 'match all but' \\n <regexp-list>

 <regexp-list> ::= <regexp> \\n <regexp-list>
                 | "<regexp>" \\n <regexp-list>
                 | <empty>\fP

.cs 4
.cs I
.fi
The default matching mode for all match files is \fB-total\fP.
See the
.BR caesar_hide_1 (LOCAL)
manual page for more information.

.SH OPTIONS

The following options are currently available:
.TP
\fB-accept\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the accept labels according to the match file \fIfile\fP.

If this option is absent, $TESTOR_DIR/src/match_accept is used as match file.
.TP
.B -all\ \ \ 
Generate all test cases that can be extracted from the complete test graph \fIctg\fP\fB.bcg\fP.
This option requires the generation of the complete test graph (see option "\fB-ctg\fP \fItestgraph\fP").
The test cases are written to files named "\fItestcase\fP\fB-N.bcg\fP", where the number \fBN\fP is incremented for each new test case.
.TP
\fB-cc\fP \fIcc-options\fP
Pass \fIoptions\fP to the C compiler when it is invoked. \fIoptions\fP is 
a list of compiler options (enclosed in quotes or double quotes). These
options are appended to the compiler options, if any, contained in the
\fB$CADP_CC\fP environment variable.
Not a default option.
.TP
\fB-count\ \ \ 
Only count and display the number of test cases, do not generate the corresponding \fBBCG\fP files.
This option requires the generation of the complete test graph (see option "\fB-ctg\fP \fItestgraph\fP").
Not a default option.
.TP
\fB-ctg\fP \fItestgraph\fP 
Force the generation of the CTG, even if on-the-fly extraction of a test case is possible.
The CTG is stored in file \fItestgraph\fP\fB.bcg\fP.
Not a default option.
.TP
.B -debug\ \ 
Do not remove intermediate files generated during the translation to C of the specification and the test purpose.
Not a default option.
.TP
.B -decorate\ \ 
Annotate transition labels of the test case and/or complete test graph to indicate if they correspond to inputs or outputs from the point of view of the tester (i.e., opposite to the point of view expressed in the input match file).
Not a default option.
.TP
.B -force
This option is passed to  \fBcombinator.open\fP, to force regeneration and compilation of LOTOS and/or C code.
See 
.BR combinator.open (LOCAL)
for more information.
Not a default option.
.TP
\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the inputs of the specification (outputs of the tester) according to the match file \fIfile\fP.
Mandatory option.
.TP
.B -monitor
Open a window for monitoring in real-time the generation of the complete test graph and each test case.
Not a default option.
.TP
\fB-otherwise\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the otherwise labels according to the match file \fIfile\fP.
Not a default option.
.TP
\fB-otherwise-matches-tau\fP
This option is passed to  \fBcombinator.open\fP, so that an "otherwise" transition of the test purpose matches also internal, hidden transitions of the specification.
See 
.BR combinator.open (LOCAL)
for more information.
Not a default option.
.TP
.B -parse\ \ \ 
Enable label parsing in the generated complete test graph and test cases.
See the
.BR bcg (LOCAL)
manual page for information about label parsing.
Default option.
.TP
\fB-purpose\fP \fIprocess-instantiation\fP
Use process \fIprocess-instantiation\fP as the principal process for the test purpose.
Default principal process is \fIMAIN\fP.
This option is only meaningful for a test purpose in LNT or LOTOS.
.TP
.B -random
Use a random number generator when choosing between different ways to solve a controllability conflict.
This option is only useful in combination with option "\fB-extract\fP".
.TP
\fB-refuse\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the refuse labels according to the match file \fIfile\fP.

If this option is absent, $TESTOR_DIR/src/match_refuse is used as match file.
.TP
.BI -seed \ N
Use \fIN\fP as seed for the random number generator.
This option is only useful in combination with options "\fB-extract\fP" and "\fB-random\fP".
.TP
.B -silent
Execute silently.
This option is passed to \fBcombinator.open\fP and \fBtestor\fP.
See
.BR combinator.open (LOCAL)
and
.BR testor (LOCAL)
for more information.
.TP
\fB-spec\fP \fIprocess-instantiation\fP
Use process \fIprocess-instantiation\fP as the principal process for the specification.
Default principal process is \fIMAIN\fP.
This option is only meaningful for a specification in LNT or LOTOS.
.TP
.B -stat\ \ \ 
Display statistics of the exploration (e.g., informations concerning the underlying Boolean equation systems) performed to compute the complete test graph.
Not a default option.
.TP
.B -timer\ \ \ \
Generate transitions managing two timers to detect acceptable (timer TNOAC) and
faulty (timer TAC) quiescence.
.TP
.B -unlock
Do not generate transitions labeled with ":DELTA:" marking quiescent states.
Not a default option.
.TP
.B -unparse
Disable label parsing in the generated complete test graph and test cases.
See the
.BR bcg (LOCAL)
manual page for information about label parsing.
Not a default option.
.TP
.B -verbose
Report in detail activities and progress, including errors and messages emitted by auxiliary tools, to the user's screen.
This option is passed to \fBcombinator.open\fP and \fBtestor\fP.
See
.BR combinator.open (LOCAL)
and
.BR testor (LOCAL)
for more information.
.TP
.B -version
Display the version number and stop.
Not a default option.

.SH OPERANDS

.PD 0
.TP 20
.IB specification .lnt
LOTOS NT specification (input)
.TP
.IB specification .lot
LOTOS specification (input)
.TP
.IB specification .lotos
LOTOS specification (input)
.TP
.IB purpose .lnt
LOTOS NT test purpose (input)
.TP
.IB purpose .lot
LOTOS test purpose (input)
.TP
.IB purpose .lotos
LOTOS test purpose (input)
.TP
.IB specification .err
detailed error messages (output)
.TP
.IB purpose .err
detailed error messages (output)
.TP
.IB ctg .bcg
complete test graph (output)

.SH EXIT STATUS
When the specification or the test purpose are erroneous, error messages are issued.
Exit status is 0 if everything is alright, 1 otherwise.

.SH AUTHORS
.P
Radu Mateescu and Wendelin Serwe.

.SH SEE ALSO
.P
.BR bcg (LOCAL),
.BR bcg_control (LOCAL),
.BR caesar_hide_1 (LOCAL),
.BR lotos.open (LOCAL),
.BR combinator.open (LOCAL),
.BR lnt.open (LOCAL),
.BR testor (LOCAL)

.P
Additional information is available from the CADP Web page located at
http://cadp.inria.fr
.P
Directives for installation are given in files \fB$CADP/INSTALLATION_*\fP.
.P
Recent changes and improvements to this software are reported
and commented in file \fB$CADP/HISTORY\fP.

.SH BUGS
.P
Please report any bug to cadp@inria.fr
