.\" @(#)testor.l - INRIA Rhone-Alpes - 2021/03/02
.TH CADP LOCAL "2021/03/02" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0
.SH NAME
testor \- compute test cases for an input/output labeled transition system
 
.SH SYNOPSIS

.B bcg_open
[\fIbcg_opt\fP]
\fIspec\fP[\fB.bcg\fP]
.RI [ cc_opt ]
.B testor
.RI [ testor_opt ]
\fItp\fP[\fB.bcg\fP|\fB.aut\fP]
\fItc\fP[\fB.bcg\fP]
.P
or:
.P
.B lotos.open
[\fIcaesar_opt\fP]
\fIspec\fP[\fB.lotos\fP]
.RI [ cc_opt ]
.B testor
.RI [ testor_opt ]
\fItp\fP[\fB.bcg\fP|\fB.aut\fP]
\fItc\fP[\fB.bcg\fP]
.P
or:
.P
.B exp.open
[\fIexp_opt\fP]
\fIspec\fP[\fB.exp\fP]
.RI [ cc_opt ]
.B testor
.RI [ testor_opt ]
\fItp\fP[\fB.bcg\fP|\fB.aut\fP]
\fItc\fP[\fB.bcg\fP]
.P
or:
.P
.B fsp.open
[\fIfsp_opt\fP]
\fIspec\fP[\fB.lts\fP]
.RI [ cc_opt ]
.B testor
.RI [ testor_opt ]
\fItp\fP[\fB.bcg\fP|\fB.aut\fP]
\fItc\fP[\fB.bcg\fP]
.P
or:
.P
.B lnt.open
[\fIlnt_opt\fP]
\fIspec\fP[\fB.lnt\fP]
.RI [ cc_opt ]
.B testor
.I testor_parameters
.RI [ testor_opt ]
\fItp\fP[\fB.bcg\fP|\fB.aut\fP]
\fItc\fP[\fB.bcg\fP]
.P
or:
.P
.B seq.open
[\fIseq_opt\fP]
\fIspec\fP[\fB.seq\fP]
.RI [ cc_opt ]
.B testor
.RI [ testor_opt ]
\fItc\fP[\fB.bcg\fP]
.P
or:
.P
.B combinator.open
[\fIcombinator_opt\fP]
\fIspec\fP[\fB.bcg\fP|\fB.lnt\fP|\fB.lot\fP|\fB.lotos\fP]
\fIpurpose\fP[\fB.bcg\fP|\fB.lnt\fP|\fB.lot\fP|\fB.lotos\fP]
.RI [ cc_opt ]
.B testor
.RI [ testor_opt ]
\fItc\fP[\fB.bcg\fP]

.SH DESCRIPTION
\fBtestor\fP extracts on the fly the test cases from an Inputs/Outputs Labeled Transition System (\s-1IOLTS\s+1) \fIspec\fP and a test purpose \fItp\fP.

The specification is either
the \s-1BCG\s+1 graph \fIspec\fP\fB.bcg\fP,
the \s-1LOTOS\s+1 specification \fIspec\fP\fB.lotos\fP,
the composition expression \fIspec\fP\fB.exp\fP,
the \s-1FSP\s+1 model \fIspec\fP\fB.lts\fP,
the \s-1LNT\s+1 model \fIspec\fP\fB.lnt\fP,
the sequence file \fIspec\fP\fB.seq\fP,
or the synchronous product of \fIspec\fP and \fIpurpose\fP.
The distinction between inputs and outputs has to be defined via the mandatory option "\fB-io\fP".

The test purpose must be a deterministic explicit LTS in the BCG or AUT format; in the absence of an extension for the test purpose, the BCG format is tried first.
The test purpose should contain at least one reachable transition labeled with "\fIACCEPT\fP".
It may contain also transitions labeled with "\fIREFUSE\fP" or "\fI*\fP".
By default, and except for the special labels "\fIACCEPT\fP", "\fIREFUSE\fP", and "\fI*\fP", labels of the test purpose are interpreted as regular expressions.
See the
.BR caesar_regexp (LOCAL)
manual page for more information about regular expressions.
For each state \fIs\fP of the test purpose with an outgoing "\fIACCEPT\fP" or "\fIREFUSE\fP" transition \fIt\fP, this transition \fIt\fP should be the only outgoing transition of \fIs\fP.
For each state \fIs\fP of the test purpose, there should be at most one outgoing transition labeled with "\fI*\fP".
Note that it is up to the user to ensure that each a label of specification is matched by at most one label (i.e., regular expression) of the test purpose (checks can be activated using option "\fB-defensive\fP").

Testor computes first a particular synchronous product between the specification and the test purpose, and then performs a depth-first search on the determinized product; the depth-first search stops when a state \fIs\fP with an outgoing transition labeled with "\fIACCEPT\fP" is reached.
If there exist sequences in the product that lead to such a state, \fBtestor\fP extracts a test case \fItc.bcg\fP in the
.BR bcg (LOCAL)
format.
A test case is an LTS where some states are tagged as "passed", "inconclusive", or "quiescent" (by means of an outgoing transition labelled ":PASS:", ":INCONCLUSIVE:", or ":DELTA", respectively).

The synchronous product is defined as follows.
The set of states of the synchronous product is the set of couples \fI<s1,s2>\fP, where \fIs1\fP is a state of the specification \fIspec\fP and \fIs2\fP is a state of the test purpose \fItp\fP.
The set of labels of the synchronous product is the set of labels of the specification, plus the labels "\fIACCEPT\fP" and "\fIREFUSE\fP" (if present in \fItp\fP).
For each state \fI<s1,s2>\fP of the product (different from the initial state), the set of outgoing transitions is computed by applying the first applicable step out of the following.

.TP 3
a)
If the test purpose contains a transition labeled with \fIl\fP from \fIs2\fP to a state \fIs2'\fP, where \fIl\fP is either "\fIACCEPT\fP" or "\fIREFUSE\fP", then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1,s2'>\fP is added to the product.

.TP
b)
If the specification contains a transition labeled with \fIl\fP from \fIs1\fP to a state \fIs1'\fP, and the test purpose contains a transition labeled with a regular expression \fIe\fP from \fIs2\fP to a state \fIs2'\fP such that \fIe\fP matches \fIl\fP, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1',s2'>\fP is added to the product.

Note that the label \fIl\fP can also be the hidden internal label "\fBi\fP": this enables to synchronize the internal transitions of the specification and the test purpose.

.TP
c)
If the specification contains a transition labeled with \fIl\fP (different from the internal label "\fBi\fP") from \fIs1\fP to a state \fIs1'\fP, and the test purpose contains a transition labeled with "\fI*\fP" from \fIs2\fP to a state \fIs2'\fP, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1',s2'>\fP is added to the product.

Note that in the test purpose, a transition labeled with "\fI*\fP" has a different semantics than a transition labeled with "\fI.*\fP" (i.e., the regular expression matching any sequence), as the latter yields a non-deterministic test purpose except if this transition is the only transition leaving its source state.

.TP 
d)
If the specification contains a transition labeled with \fIl\fP from \fIs1\fP to a state \fIs1'\fP, then a transition labeled with \fIl\fP from \fI<s1,s2>\fP to \fI<s1',s2>\fP is added to the product.

Note that this rules corresponds to an implicit completion of the test purpose with a transition labeled with "\fI*\fP" from \fIs2\fP to \fIs2\fP.
.P

.SH B\&C\&G FORMAT
See the
.BR bcg (LOCAL)
manual page.

.SH MATCH FILE FORMAT

Each match file characterizes a set of actions by means of a set of regular expressions (according to the definition given in the manual page of the \s-1POSIX\s+1
.BR regexp (LOCAL)),
according to the following grammar:
.ne 6v
.nf
.cs 4 23
.cs I 23

 \fB<i/o-match-file> ::= 'input' | 'output' \\n <regexp-list>

 \fB<match-file> ::= 'match' | 'match all but' \\n <regexp-list>

 <regexp-list> ::= <regexp> \\n <regexp-list>
                 | "<regexp>" \\n <regexp-list>
                 | <empty>\fP

.cs 4
.cs I
.fi
.P
The default matching mode for all match files is \fB-gate\fP.
See the
.BR caesar_hide_1 (LOCAL)
manual page for more information.

.SH OPTIONS
The options \fIbcg_opt\fP, if any, are passed to
.BR bcg_lib (LOCAL).
.P
The options \fIcaesar_opt\fP, if any, are passed to
.BR caesar (LOCAL)
and to
.BR caesar.adt (LOCAL).
.P
The options \fIexp_opt\fP, if any, are passed to
.BR exp.open (LOCAL).
.P
The options \fIfsp_opt\fP, if any, are passed to
.BR fsp.open (LOCAL).
.P
The options \fIlnt_opt\fP, if any, are passed to
.BR lnt.open (LOCAL).
.P
The options \fseq_opt\fP, if any, are passed to
.BR seq.open (LOCAL).
.P
The options \fIcombinator_opt\fP, if any, are passed to
.BR combinator.open (LOCAL).
.P
The options \fIcc_opt\fP, if any,  are passed to
.BR cc (1).
.P
The options \fItestor_opt\fP, if any, are passed to the \fBtestor\fP program.

.SH TESTOR OPTIONS
The following \fItestor_opt\fP are currently available:
.TP
\fB-accept\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the accept labels according to the match file \fIfile\fP.
If this option is absent, the file \fB$TESTOR_DIR/src/match_accept\fP is used in gate matching mode. 
This option is only meaningful in combination with option \fI-product\fP.
Not a default option.
.TP
\fB-alternatives\fP \fIold_alternatives\fP \fInew_alternatives\fP
Compute file \fInew_alternatives\fP by removing from \fIold_alternatives\fP all the transitions that are also present in the extracted test case.
Not a default option.
.TP
\fB-csg\fP | \fB-ctg\fP\ \ \ \ \ \ 
Generate a complete test graph (rather than a single test case).
Not a default option.
.TP
.B -decorate\ \ \ \ \ \ 
Annotate transition labels of the complete test graph to indicate if they correspond to inputs or outputs from the point of view of the tester (i.e., opposite to the point of view expressed in the input match file).
This option also changes how states are tagged: instead of adding outgoing transitions, the tag is added to incoming transitions.
Not a default option.
.TP
.B -defensive\ \ \ \ \ \ 
When computing the product, check that a label of the specification is not matched by several transitions of the test purpose.
This option is not meaningful in combination with option \fI-product\fP.
Not a default option.

Note that by default the first matching transition of the test purpose is chosen, but the inspection order of the transitions is undefined.
.TP
.B -extract\ \ \ 
Directly generate a test case (rather than a complete test graph), by on-the-fly enforcement of the controllability constraints described in [JJ05, definition 5].
Default option.
.TP
.B -fair\ \ \ \ 
When computing the suspension automaton, consider a fair semantics for quiescence, i.e., a state is quiescent iff it has no outgoing transition labeled with an output of the specification.
This option is only meaningful in combination with option \fI-suspension\fP.
Not a default option.
.TP
.B -gate\ \ \ \ \ \ 
Use \fIgate matching\fP when matching a label of the test purpose (interpreted as a regular expression) against a label of the specification.
This option is not meaningful in combination with option \fI-product\fP.
Default option.
.TP
.B -interactive\ \ \ 
Directly execute a test case (generated on the fly), connecting to an implementation under test by writing labels corresponding to inputs of the specification to the standard error and reading labels corresponding to output of the specification from standard input.
Each label corresponds to one line of input or output, without the terminating newline character.
If this option is present, no BCG file is generated, and there should not be a corresponding argument on the command line.
Not a default option.
.TP
\fB-io\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the input/output status of labels of the specification, according to the input/output match file \fIfile\fP.
Mandatory option.
.TP
\fB-otherwise-matches-tau\fP
If this option is present, an "otherwise" transition of the test purpose matches also internal, hidden transitions of the specification.
Concretely, this option removes the constraint "\fIl\fP different from the internal label \fBi\fP" from step c) above.
Not a default option.
.TP
.B -partial\ \ \ \ \ \ 
Use \fIpartial matching\fP when matching a label of the test purpose (interpreted as a regular expression) against a label of the specification.
This option is not meaningful in combination with option \fI-product\fP.
Not a default option.
.TP
\fB-prefix\fP \fIfile\fP[\fB.bcg\fP]
Force the extracted test case to include the initial prefix \fIfile\fP\fB.bcg\fP.
This option requires the synchronous product to be deterministic and without interal transitions; a typical use case is the extraction of a test case from a complete test graph (which satisfies these preconditions by construction), together with option \fB-product\fP and match files specifying ":PASS:" as "\fIACCEPT\fP" and ":INCONCLUSIVE:" as "\fIREFUSE\fP".
This option is incompatible with option \fB-ctg\fP.
Not a default option. 
.TP
.B -product\ \ \ 
Do not compute the synchronous product.
This option requires the presence of options \fI-accept\fP and \fI-refuse\fP.
Not a default option. 
.TP
\fB-refuse\fP [\fB-gate\fP|\fB-partial\fP|\fB-total\fP] \fIfile\fP
Specify the refuse labels according to the match file \fIfile\fP.
If this option is absent, the file \fB$TESTOR_DIR/src/match_refuse\fP is used in gate matching mode. 
This option is only meaningful in combination with option \fI-product\fP.
Not a default option.
.TP
.B -self\ \ \ 
Do not report implicit completion of the states of the test purpose.
This option is only meaningful in combination with option \fI-verbose\fP.
Default option.
.TP
.B -silent\ \ \ 
Do not report activities and progress to the user's screen.
Default option.
.TP
.B -stat\ \ \ \ \ \ \ \ \ 
Display statistics of the exploration (e.g., informations concerning the underlying Boolean equation systems).
Not a default option.
.TP
.B -suspension\ \ \ \ \ \ \ \ \ 
Compute only the suspension automaton.
Does not compute a CTG or extract a test case.
Not a default option.
.TP
.B -taufree\ \ \ \ 
Use an optimized exploration algorithm, which handles internal transitions as normal transitions.
This option reduces both runtime and memory requirements, but is only correct if there are no internal transitions.
Not a default option.
.TP
.B -timer\ \ \ \  
Generate transitions managing two timers to detect acceptable (timer TNOAC) and faulty (timer TAC) quiescence.
.TP
.B -total\ \ \ \ \ \ 
Use \fItotal matching\fP when matching a label of the test purpose (interpreted as a regular expression) against a label of the specification.
This option is not meaningful in combination with option \fI-product\fP.
Not a default option.
.TP
.B -unlock\ \ \ 
Do not generate transitions labeled with ":DELTA:" marking quiescent states.
Not a default option.
.TP
.BR -verbose\ \ \ 
Report in detail activities and progress to the user's screen.
Not a default option.
.TP
.BR -version\ \ \ 
Display the version number and stop.
Not a default option.

.SH AUTHORS
Radu Mateescu and Wendelin Serwe.

.SH BIBLIOGRAPHY

.TP 8
[JJ05]
Claude Jard and Thierry Jeron.
TGV: theory, principles and algorithms.
International Journal on Software Tools for Technology Transfer 7(4):297-315, August 2005.

.SH SEE ALSO
.P
.BR bcg (LOCAL),
.BR caesar_hide_1 (LOCAL),
.BR caesar_regexp (LOCAL),
.BR extract_tests (LOCAL)

.P
Additional information is available from the CADP Web page located at
http://cadp.inria.fr
.P
Directives for installation are given in files \fB$CADP/INSTALLATION_*\fP.
.P
Recent changes and improvements to this software are reported
and commented in file \fB$CADP/HISTORY\fP.

.SH BUGS
Please report any bug to cadp@inria.fr


